package javax.swing;

import javax.accessibility.AccessibleRole;
import javax.accessibility.AccessibleState;
import java.awt.event.ItemListener;
import java.awt.AWTEvent;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.EventQueue;
import java.awt.event.ItemEvent;
import javax.accessibility.AccessibleContext;
import java.io.IOException;
import java.io.ObjectOutputStream;
import javax.swing.plaf.ButtonUI;
import javax.accessibility.Accessible;

public class JToggleButton extends AbstractButton implements Accessible
{
    private static final String uiClassID = "ToggleButtonUI";
    
    public JToggleButton() {
        this(null, null, false);
    }
    
    public JToggleButton(Icon icon) {
        this(null, icon, false);
    }
    
    public JToggleButton(Icon icon, boolean b) {
        this(null, icon, b);
    }
    
    public JToggleButton(String s) {
        this(s, null, false);
    }
    
    public JToggleButton(String s, boolean b) {
        this(s, null, b);
    }
    
    public JToggleButton(Action action) {
        this();
        this.setAction(action);
    }
    
    public JToggleButton(String s, Icon icon) {
        this(s, icon, false);
    }
    
    public JToggleButton(String s, Icon icon, boolean selected) {
        this.setModel(new JToggleButton.ToggleButtonModel());
        this.model.setSelected(selected);
        this.init(s, icon);
    }
    
    @Override
    public void updateUI() {
        this.setUI((ButtonUI)UIManager.getUI(this));
    }
    
    @Override
    public String getUIClassID() {
        return "ToggleButtonUI";
    }
    
    @Override
    boolean shouldUpdateSelectedStateFromAction() {
        return true;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        if (this.getUIClassID().equals("ToggleButtonUI")) {
            int n = (byte)(JComponent.getWriteObjCounter(this) - 1);
            JComponent.setWriteObjCounter(this, (byte)n);
            if (n == 0 && this.ui != null) {
                this.ui.installUI(this);
            }
        }
    }
    
    @Override
    protected String paramString() {
        return super.paramString();
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new JToggleButton.AccessibleJToggleButton();
        }
        return this.accessibleContext;
    }
    
    public static class ToggleButtonModel extends DefaultButtonModel
    {
        @Override
        public boolean isSelected() {
            return (this.stateMask & 0x2) != 0x0;
        }
        
        @Override
        public void setSelected(boolean selected) {
            ButtonGroup group = this.getGroup();
            if (group != null) {
                group.setSelected(this, selected);
                selected = group.isSelected(this);
            }
            if (this.isSelected() == selected) {
                return;
            }
            if (selected) {
                this.stateMask |= 0x2;
            }
            else {
                this.stateMask &= 0xFFFFFFFD;
            }
            this.fireStateChanged();
            this.fireItemStateChanged(new ItemEvent(this, 701, this, this.isSelected() ? 1 : 2));
            return;
        }
        
        @Override
        public void setPressed(boolean b) {
            if (this.isPressed() == b || !this.isEnabled()) {
                return;
            }
            if (!b && this.isArmed()) {
                this.setSelected(!this.isSelected());
            }
            if (b) {
                this.stateMask |= 0x4;
            }
            else {
                this.stateMask &= 0xFFFFFFFB;
            }
            this.fireStateChanged();
            if (!this.isPressed() && this.isArmed()) {
                int modifiers = 0;
                AWTEvent currentEvent = EventQueue.getCurrentEvent();
                if ((currentEvent instanceof InputEvent)) {
                    modifiers = ((InputEvent)currentEvent).getModifiers();
                }
                else if ((currentEvent instanceof ActionEvent)) {
                    modifiers = ((ActionEvent)currentEvent).getModifiers();
                }
                this.fireActionPerformed(new ActionEvent(this, 1001, this.getActionCommand(), EventQueue.getMostRecentEventTime(), modifiers));
            }
            return;
        }
    }
    
    protected class AccessibleJToggleButton extends AbstractButton.AccessibleAbstractButton implements ItemListener
    {
        public AccessibleJToggleButton() {
            JToggleButton.this.addItemListener(this);
        }
        
        @Override
        public void itemStateChanged(ItemEvent itemEvent) {
            JToggleButton toggleButton = (JToggleButton)itemEvent.getSource();
            if (JToggleButton.this.accessibleContext != null) {
                if (toggleButton.isSelected()) {
                    JToggleButton.this.accessibleContext.firePropertyChange("AccessibleState", null, AccessibleState.CHECKED);
                }
                else {
                    JToggleButton.this.accessibleContext.firePropertyChange("AccessibleState", AccessibleState.CHECKED, null);
                }
            }
        }
        
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.TOGGLE_BUTTON;
        }
    }
}
